//---------------------------------------------------------------------------
//
//                      WWW.NATUREWIZARD.COM
//                      ~~~~~~~~~~~~~~~~~~~~
//
//              Author      : Dr. Heinz Seyringer
//              Project     : Tutorial 1 - Textures
//              Description : This project demonstrates how to use textures
//
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "MAIN.h"
#include "gl/glut.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TFormMain *FormMain;
//---------------------------------------------------------------------------
__fastcall TFormMain::TFormMain(TComponent* Owner)
    : TForm(Owner)
{
    Application->OnIdle = IdleLoop;   // Define idle function
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::SetupLighting()
{
    GLfloat MaterialAmbient[] = {0.5, 0.5, 0.5, 1.0};
    GLfloat MaterialDiffuse[] = {1.0, 1.0, 1.0, 1.0};
    GLfloat MaterialSpecular[] = {1.0, 1.0, 1.0, 1.0};
    GLfloat MaterialShininess[] = {50.0};
    GLfloat AmbientLightPosition[] = {0.5, 1.0, 1.0, 0.0};
    GLfloat LightAmbient[] = {0.5, 0.5, 0.5, 1.0};

    glMaterialfv(GL_FRONT, GL_AMBIENT, MaterialAmbient);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, MaterialDiffuse);
    glMaterialfv(GL_FRONT, GL_SPECULAR, MaterialSpecular);
    glMaterialfv(GL_FRONT, GL_SHININESS, MaterialShininess);
    glLightfv(GL_LIGHT0, GL_POSITION, AmbientLightPosition);
    glLightModelfv(GL_LIGHT_MODEL_AMBIENT, LightAmbient);

    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
    glEnable(GL_COLOR_MATERIAL);
    glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
    glShadeModel(GL_SMOOTH);
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::SetupTexture()
{
    const int texturesize1=256;
    const int texturesize2=256;

    bitmap = new Graphics::TBitmap;
    bitmap->LoadFromFile("texture07.bmp");
    GLubyte bits[texturesize1][texturesize1][4];
    for(int i = 0; i < texturesize1; i++)
    {
    	for(int j = 0; j < texturesize1; j++)
        {
            bits[i][j][0]= (GLbyte)GetRValue(bitmap->Canvas->Pixels[i][j]);
            bits[i][j][1]= (GLbyte)GetGValue(bitmap->Canvas->Pixels[i][j]);
            bits[i][j][2]= (GLbyte)GetBValue(bitmap->Canvas->Pixels[i][j]);
            bits[i][j][3]= (GLbyte)255;
        }
    }

    glPixelStorei(GL_UNPACK_ALIGNMENT, 4);   // specify memory alignment
    glGenTextures(1, &texture1);
    glBindTexture(GL_TEXTURE_2D, texture1);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, texturesize1, texturesize1, 0, GL_RGBA, GL_UNSIGNED_BYTE, bits);



    bitmap->LoadFromFile("texture02.bmp");
    GLubyte bits2[texturesize2][texturesize2][4];
    for(int i = 0; i < texturesize2; i++)
    {
    	for(int j = 0; j < texturesize2; j++)
        {
            bits2[i][j][0]= (GLbyte)GetRValue(bitmap->Canvas->Pixels[i][j]);
            bits2[i][j][1]= (GLbyte)GetGValue(bitmap->Canvas->Pixels[i][j]);
            bits2[i][j][2]= (GLbyte)GetBValue(bitmap->Canvas->Pixels[i][j]);
            bits2[i][j][3]= (GLbyte)255;
        }
    }

    glPixelStorei(GL_UNPACK_ALIGNMENT, 4);   // specify memory alignment
    glGenTextures(1, &texture2);
    glBindTexture(GL_TEXTURE_2D, texture2);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, texturesize2, texturesize2, 0, GL_RGBA, GL_UNSIGNED_BYTE, bits2);

    glEnable(GL_TEXTURE_2D);
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);

}
//---------------------------------------------------------------------------
void __fastcall TFormMain::IdleLoop(TObject*, bool& done)
{
    done = false;               // request more idle time from the system
    RenderGLScene();
    SwapBuffers(hdc);
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::FormCreate(TObject *Sender)
{
    alpha = 0.0;                   // set y-rotation angle to 0
    beta  = 0.0;                   // set x-rotation angle to 0
    hdc = GetDC(Handle);           // get device context from main window
    SetPixelFormatDescriptor();
    hrc = wglCreateContext(hdc);   // use device context to create
                                   // a rendering context
    if(hrc == NULL)
    	ShowMessage("Creating Rendering Context failed.");
    wglMakeCurrent(hdc,hrc);       // tell windows to use hdc and rdc
    if(wglMakeCurrent(hdc, hrc) == false)
    	ShowMessage("MakeCurrent of rendering context failed.");
    w = ClientWidth;               // width of main windows client area
    h = ClientHeight;              // height of main windows client area

    glEnable(GL_DEPTH_TEST);
    glDisable(GL_CULL_FACE);

    glClearColor(0.0f, 0.0f, 0.0f, 1.0f);  // clear background to black
    glClearDepth(100.0);                   // set depth buffer to the most distant value
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    SetupLighting();
    SetupTexture();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::SetPixelFormatDescriptor()
{
    PIXELFORMATDESCRIPTOR pfd = {
    	sizeof(PIXELFORMATDESCRIPTOR),       // size of the structure
        1,                                   // structure version
        PFD_DRAW_TO_WINDOW |                 // draw directly to window
                                             // (not to a bitmap file)
        PFD_SUPPORT_OPENGL |                 // allow DC to support opengl calls
        PFD_DOUBLEBUFFER,                    // use double buffer
        PFD_TYPE_RGBA,                       // use RGBA color mode (A = alpha)
        24,                                  // use 24 bit color
        0,0,0,0,0,0,0,0,0,0,0,0,0,           // use default bitplanes
        32,                                  // use 32 bit z buffer
        0,0,                                 // not (yet) used
        PFD_MAIN_PLANE,                      // draw to main plane
        0,0,0,                               // different mask bits
    };
    PixelFormat = ChoosePixelFormat(hdc, &pfd);  // choose and set the
    SetPixelFormat(hdc, PixelFormat, &pfd);      // appropriate pixelformat
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::FormResize(TObject *Sender)
{

    GLfloat Range = 100.0;  // numerical range on the 3 axes

    w = ClientWidth;      // check new client width of the main window
    h = ClientHeight;     // check new client height of the main window

    if(h == 0)            // prevent a division by zero, by making sure that
    	h = 1;            // windows height is at least 1

    glViewport(0, 0, w, h);         // reset viewport


    glMatrixMode(GL_PROJECTION);    // add perspective to scene
    glLoadIdentity();               // restore matrix to original state

    if (w <= h)
        glOrtho (-Range, Range, -Range*h/w, Range*h/w, -Range, Range);
    else
	     glOrtho (-Range*w/h, Range*w/h, -Range, Range, -Range, Range);

    glMatrixMode(GL_MODELVIEW);
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::RenderGLScene()
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    DrawObjects();
    glFlush();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::DrawObjects()
{

   // rendering functions
   alpha+=0.1;
   glLoadIdentity();
   glRotatef(alpha,1.0,1.0,0.0);
   glBindTexture(GL_TEXTURE_2D, texture1);
   glTranslatef(-40.0,-30.0,10.0);
   glutSolidTeapot(40.0);

   glBindTexture(GL_TEXTURE_2D, texture2);
   glLoadIdentity();
   glTranslatef(40.0,20.0,10.0);
   glBegin(GL_QUADS);                           // draw square
      glTexCoord2f(0.0,0.0);
      glVertex3f(-50.0f,50.0f, -50.0f);
      glTexCoord2f(1.0,0.0);
      glVertex3f( -50.0f, -50.0f, -50.0f);
      glTexCoord2f(1.0,1.0);
      glVertex3f( 50.0f, -50.0f, -50.0f);
      glTexCoord2f(0.0,1.0);
      glVertex3f( 50.0f, 50.0f, -50.0f);
   glEnd();

}
//---------------------------------------------------------------------------
void __fastcall TFormMain::FormPaint(TObject *Sender)
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    DrawObjects();
    glFlush();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::FormDestroy(TObject *Sender)
{
    delete bitmap;            // delete texture bitmap
    wglMakeCurrent(NULL, NULL);
    wglDeleteContext(hrc);
    glDeleteTextures(1, &texture1);
    glDeleteTextures(1, &texture2);
}

